*  Program...........: CHECKDD.PRG
*  Author............: Steven Kern
*  Project...........: DDICT25
*  Created...........: 01/28/94
*  Copyright.........: (c) Steven Kern, 1994
*) Description.......: checks existing Datadict for "standard" Code Book
*                    : fields and adds any that are missing, such as the
*                    : new field LSHOW. This preserves your own
*                    : Datadict table structure by only appending those
*                    : fields which are missing.
*  Calling Samples...: DO Checkdd
*  Parameter List....: 
*  Major change list.: 

PRIVATE ALL LIKE j*

WAIT WINDOW NOWAIT "Checking Datadict..."

*-- logical variable for adding fields
STORE .F. TO jlNewFields

*-- Array for storing tags from original Datadict
DIMENSION jaTags[1,2]

*-- Row counter for jaTags
STORE 0 TO jnTags

*-- Set the path, saving the original path to return to
jcOrigPath = CURDIR()
SET DEFAULT TO (lcDDPath)  && the \DBFS subdirectory

*-- Build an array of the necessary fields to check against Datadict
*   must be an array like AFIELDS() creates of 4 elements:
*   FIELD_NAME, FIELD_TYPE, FIELD_LEN, FIELD_DEC
*   Rather than declaring each element 'by hand', CREATE a temporary
*   table, and do AFIELDS(temptable). Basically, I'm lazy, and didn't
*   want to type all that stuff in <g>! 

*[SK CHANGE 05/23/94 
*   Instead of CREATing a TABLE, just CREATE a CURSOR! No need
*   for extra disk access

CREATE CURSOR lqTempDBF ;
   (FIELD_NAME C(10), ;
   FIELD_TYPE C(1),  ;
   FIELD_LEN  N(3),  ;
   FIELD_DEC  N(3),  ;
   DBF_NAME   C(8),  ;
   LINDEX     L(1),  ;
   INDEX_KEY  M(10), ;
   CTAG       C(20), ;
   LSHOW      L(1))

*-- Get lqTempDBF's fields into an array
=AFIELDS(jaCheckIt)

*[END CHANGE 05/23/94

*-- Get Datadict's fields into an array
SELECT Datadict
=AFIELDS(jaDatadict)

*-- Now Check for missing fields using ASCAN() 
FOR i = 1 TO 9
   jnMissing=ASCAN(jaDatadict,jaCheckIt[i,1],1)
   IF jnMissing = 0
      *-- reDIMENSION jaDatadict and add the missing fields
      lnRows = ALEN(jaDatadict,1)+1
      lnCols = ALEN(jaDatadict,2)
      DIMENSION jaDatadict[lnRows,lnCols]
      jaDatadict[lnRows,1] = jaCheckIt[i,1]
      jaDatadict[lnRows,2] = jaCheckIt[i,2]
      jaDatadict[lnRows,3] = jaCheckIt[i,3]
      jaDatadict[lnRows,4] = jaCheckIt[i,4]      
      *-- Set flag for new fields
      STORE .T. TO jlNewFields
   ENDIF jnMissing = 0
ENDFOR i = 1 TO 10

*-- If jlNewFields, create a new table from jaDatadict and
*   APPEND from Datadict
IF jlNewFields
  IF !AreUSure("Necessary fields are missing from Datadict.dbf. The table " + ;
      "will be modified, and a copy of the original saved under DataOld.dbf. " + ;
      " Do you wish to continue?")
      RETURN .F.
  ENDIF !AreUSure()
   
    *  DO Alert.spr WITH "Modifying Datadict! A copy of the original table " + ;
    *   "is saved under the name DataOld.dbf"  
  WAIT WINDOW NOWAIT "Copying..."    
  COPY TO Dataold WITH PRODUCTION
     
  *-- Get tags from original Datadict and store in array jaTAgs
  DO Tagit
  
  *-- Now, create a new table from jaDatadict and APPEND from Datadict
  jcTemp    = SYS(3)
  jcTempDBF = jcTemp+".dbf"
  jcTempFPT = jcTemp+".fpt"

  WAIT WINDOW NOWAIT "Creating new table..."      
  CREATE TABLE (jcTempDBF) FROM ARRAY jaDatadict
  APPEND FROM Datadict

  *[CHANGE 05/23/94 Streamline using COPY TO rather than ERASE/RENAME
  
  *-- OK, we're already using (jctempdbf), but first close Datadict  
  USE IN Datadict
  COPY TO Datadict  && don't have to erase the  .cdx file <g>

  *-- Now, ERASE the temp files:  
  WAIT WINDOW NOWAIT "Erasing old files..."      
  USE      && Table must be closed to erase
  ERASE (jcTempDBF)
  ERASE (jcTempFPT)
  
  *[END CHANGE 05/23/94
  
  *-- Reindex Datadict
  =TablOpen("Datadict","Datadict","","EXCLUSIVE")
  SELECT Datadict

  *-- Restore original tags
  *   Standard Code Book tags are written in calling module
  FOR i = 1 to jnTags
    *-- Store array vars to memory vars to prevent error
    INDEX ON &jaTags[i,2] TAG &jaTags[i,1]
  ENDFOR       

ENDIF jlNewFields

*-- Restore original directory
SET DEFAULT TO (jcOrigPath)  
WAIT CLEAR  
RETURN

************************************************
PROCEDURE TagIt     
************************************************

*  Procedure.........: TagIt     
*  Author............: Steven Kern
*  Project...........: DDICT25
*  Created...........: 01/29/94
*  Copyright.........: (c) Steven Kern, 1994
*) Description.......: Preserves index tags for old Datadict
*  Calling Samples...: DO Tagit
*  Parameter List....: 
*  Major change list.:  
 
SELECT Datadict
FOR i = 1 TO 256
   IF EMPTY(TAG(i))
      EXIT                     && There are no tags
   ELSE
      *-- Increment Row Counter jnTags
      jnTags = jnTags + 1
      lcDbfTags = TAG(i)
      lcKey		= KEY(i)		&& Store the index key

      *-- accommodate Filtered Indexes
      lcFilter = SYS(2021,i)
      IF !EMPTY(lcFilter)
         lcKey = lcKey + " FOR " + lcFilter
      ENDIF !EMPTY(lcFilter)

    *[Change 05/18/94  Add new feature from 2.6, DESCENDING()
    *  Be sure version 2.6 is running first
      IF '2.6' $ VERSION()
        IF DESCENDING(i)
          lcKey = lcKey + " DESCENDING"      
        ENDIF DESCENDING(i)
      ENDIF '2.6' $ VERSION()
    *[END CHANGE

      *-- reDIMENSION the array if jnTags > 1
      IF jnTags # 1
         DIMENSION jaTags[jnTags,2]
      ENDIF jnTags # 1

      *-- Store the index tag in the array jnTags, 1st element
      jaTags[jnTags,1] = lcDbfTags

      *-- Store the index key in the array jnTags, 2nd element
      jaTags[jnTags,2] = lcKey

   ENDIF EMPTY(TAG(i))      
ENDFOR i = 1 TO 256

RETURN
